#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.util."""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'


import datetime
import random
import sys
import types
import unittest

from protorpc import test_util
from protorpc import util


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = util


class PadStringTest(test_util.TestCase):

  def testPadEmptyString(self):
    self.assertEquals(' ' * 512, util.pad_string(''))

  def testPadString(self):
    self.assertEquals('hello' + (507 * ' '), util.pad_string('hello'))

  def testPadLongString(self):
    self.assertEquals('x' * 1000, util.pad_string('x' * 1000))


class UtilTest(test_util.TestCase):

  def testDecoratedFunction_LengthZero(self):
    @util.positional(0)
    def fn(kwonly=1):
      return [kwonly]
    self.assertEquals([1], fn())
    self.assertEquals([2], fn(kwonly=2))
    self.assertRaisesWithRegexpMatch(TypeError,
                                     r'fn\(\) takes at most 0 positional '
                                     r'arguments \(1 given\)',
                                     fn, 1)

  def testDecoratedFunction_LengthOne(self):
    @util.positional(1)
    def fn(pos, kwonly=1):
      return [pos, kwonly]
    self.assertEquals([1, 1], fn(1))
    self.assertEquals([2, 2], fn(2, kwonly=2))
    self.assertRaisesWithRegexpMatch(TypeError,
                                     r'fn\(\) takes at most 1 positional '
                                     r'argument \(2 given\)',
                                     fn, 2, 3)

  def testDecoratedFunction_LengthTwoWithDefault(self):
    @util.positional(2)
    def fn(pos1, pos2=1, kwonly=1):
      return [pos1, pos2, kwonly]
    self.assertEquals([1, 1, 1], fn(1))
    self.assertEquals([2, 2, 1], fn(2, 2))
    self.assertEquals([2, 3, 4], fn(2, 3, kwonly=4))
    self.assertRaisesWithRegexpMatch(TypeError,
                                     r'fn\(\) takes at most 2 positional '
                                     r'arguments \(3 given\)',
                                     fn, 2, 3, 4)

  def testDecoratedMethod(self):
    class MyClass(object):
      @util.positional(2)
      def meth(self, pos1, kwonly=1):
        return [pos1, kwonly]
    self.assertEquals([1, 1], MyClass().meth(1))
    self.assertEquals([2, 2], MyClass().meth(2, kwonly=2))
    self.assertRaisesWithRegexpMatch(TypeError,
                                     r'meth\(\) takes at most 2 positional '
                                     r'arguments \(3 given\)',
                                     MyClass().meth, 2, 3)

  def testDefaultDecoration(self):
    @util.positional
    def fn(a, b, c=None):
      return a, b, c
    self.assertEquals((1, 2, 3), fn(1, 2, c=3))
    self.assertEquals((3, 4, None), fn(3, b=4))
    self.assertRaisesWithRegexpMatch(TypeError,
                                     r'fn\(\) takes at most 2 positional '
                                     r'arguments \(3 given\)',
                                     fn, 2, 3, 4)

  def testDefaultDecorationNoKwdsFails(self):
    def fn(a):
      return a
    self.assertRaisesRegexp(
        ValueError,
        'Functions with no keyword arguments must specify max_positional_args',
        util.positional, fn)

  def testDecoratedFunctionDocstring(self):
    @util.positional(0)
    def fn(kwonly=1):
      """fn docstring."""
      return [kwonly]
    self.assertEquals('fn docstring.', fn.__doc__)


class AcceptItemTest(test_util.TestCase):

  def CheckAttributes(self, item, main_type, sub_type, q=1, values={}, index=1):
    self.assertEquals(index, item.index)
    self.assertEquals(main_type, item.main_type)
    self.assertEquals(sub_type, item.sub_type)
    self.assertEquals(q, item.q)
    self.assertEquals(values, item.values)

  def testParse(self):
    self.CheckAttributes(util.AcceptItem('*/*', 1), None, None)
    self.CheckAttributes(util.AcceptItem('text/*', 1), 'text', None)
    self.CheckAttributes(util.AcceptItem('text/plain', 1), 'text', 'plain')
    self.CheckAttributes(
      util.AcceptItem('text/plain; q=0.3', 1), 'text', 'plain', 0.3,
      values={'q': '0.3'})
    self.CheckAttributes(
      util.AcceptItem('text/plain; level=2', 1), 'text', 'plain',
      values={'level': '2'})
    self.CheckAttributes(
      util.AcceptItem('text/plain', 10), 'text', 'plain', index=10)

  def testCaseInsensitive(self):
    self.CheckAttributes(util.AcceptItem('Text/Plain', 1), 'text', 'plain')

  def testBadValue(self):
    self.assertRaises(util.AcceptError,
                      util.AcceptItem, 'bad value', 1)
    self.assertRaises(util.AcceptError,
                      util.AcceptItem, 'bad value/', 1)
    self.assertRaises(util.AcceptError,
                      util.AcceptItem, '/bad value', 1)

  def testSortKey(self):
    item = util.AcceptItem('main/sub; q=0.2; level=3', 11)
    self.assertEquals((False, False, -0.2, False, 11), item.sort_key)

    item = util.AcceptItem('main/*', 12)
    self.assertEquals((False, True, -1, True, 12), item.sort_key)

    item = util.AcceptItem('*/*', 1)
    self.assertEquals((True, True, -1, True, 1), item.sort_key)

  def testSort(self):
    i1 = util.AcceptItem('text/*', 1)
    i2 = util.AcceptItem('text/html', 2)
    i3 = util.AcceptItem('text/html; q=0.9', 3)
    i4 = util.AcceptItem('text/html; q=0.3', 4)
    i5 = util.AcceptItem('text/xml', 5)
    i6 = util.AcceptItem('text/html; level=1', 6)
    i7 = util.AcceptItem('*/*', 7)
    items = [i1, i2 ,i3 ,i4 ,i5 ,i6, i7]
    random.shuffle(items)
    self.assertEquals([i6, i2, i5, i3, i4, i1, i7], sorted(items))

  def testMatchAll(self):
    item = util.AcceptItem('*/*', 1)
    self.assertTrue(item.match('text/html'))
    self.assertTrue(item.match('text/plain; level=1'))
    self.assertTrue(item.match('image/png'))
    self.assertTrue(item.match('image/png; q=0.3'))

  def testMatchMainType(self):
    item = util.AcceptItem('text/*', 1)
    self.assertTrue(item.match('text/html'))
    self.assertTrue(item.match('text/plain; level=1'))
    self.assertFalse(item.match('image/png'))
    self.assertFalse(item.match('image/png; q=0.3'))

  def testMatchFullType(self):
    item = util.AcceptItem('text/plain', 1)
    self.assertFalse(item.match('text/html'))
    self.assertTrue(item.match('text/plain; level=1'))
    self.assertFalse(item.match('image/png'))
    self.assertFalse(item.match('image/png; q=0.3'))

  def testMatchCaseInsensitive(self):
    item = util.AcceptItem('text/plain', 1)
    self.assertTrue(item.match('tExt/pLain'))

  def testStr(self):
    self.assertHeaderSame('*/*', str(util.AcceptItem('*/*', 1)))
    self.assertHeaderSame('text/*', str(util.AcceptItem('text/*', 1)))
    self.assertHeaderSame('text/plain',
                          str(util.AcceptItem('text/plain', 1)))
    self.assertHeaderSame('text/plain; q=0.2',
                          str(util.AcceptItem('text/plain; q=0.2', 1)))
    self.assertHeaderSame(
        'text/plain; q=0.2; level=1',
        str(util.AcceptItem('text/plain; level=1; q=0.2', 1)))

  def testRepr(self):
    self.assertEquals("AcceptItem('*/*', 1)", repr(util.AcceptItem('*/*', 1)))
    self.assertEquals("AcceptItem('text/plain', 11)",
                      repr(util.AcceptItem('text/plain', 11)))

  def testValues(self):
    item = util.AcceptItem('text/plain; a=1; b=2; c=3;', 1)
    values = item.values
    self.assertEquals(dict(a="1", b="2", c="3"), values)
    values['a'] = "7"
    self.assertNotEquals(values, item.values)


class ParseAcceptHeaderTest(test_util.TestCase):

  def testIndex(self):
    accept_header = """text/*, text/html, text/html; q=0.9,
                       text/xml,
                       text/html; level=1, */*"""
    accepts = util.parse_accept_header(accept_header)
    self.assertEquals(6, len(accepts))
    self.assertEquals([4, 1, 3, 2, 0, 5], [a.index for a in accepts])


class ChooseContentTypeTest(test_util.TestCase):

  def testIgnoreUnrequested(self):
    self.assertEquals('application/json',
                      util.choose_content_type(
                        'text/plain, application/json, */*',
                        ['application/X-Google-protobuf',
                         'application/json'
                        ]))

  def testUseCorrectPreferenceIndex(self):
    self.assertEquals('application/json',
                      util.choose_content_type(
                        '*/*, text/plain, application/json',
                        ['application/X-Google-protobuf',
                         'application/json'
                        ]))

  def testPreferFirstInList(self):
    self.assertEquals('application/X-Google-protobuf',
                      util.choose_content_type(
                        '*/*',
                        ['application/X-Google-protobuf',
                         'application/json'
                        ]))

  def testCaseInsensitive(self):
    self.assertEquals('application/X-Google-protobuf',
                      util.choose_content_type(
                        'application/x-google-protobuf',
                        ['application/X-Google-protobuf',
                         'application/json'
                        ]))


class GetPackageForModuleTest(test_util.TestCase):

  def setUp(self):
    self.original_modules = dict(sys.modules)

  def tearDown(self):
    sys.modules.clear()
    sys.modules.update(self.original_modules)

  def CreateModule(self, name, file_name=None):
    if file_name is None:
      file_name = '%s.py' % name
    module = types.ModuleType(name)
    sys.modules[name] = module
    return module

  def assertPackageEquals(self, expected, actual):
    self.assertEquals(expected, actual)
    if actual is not None:
      self.assertTrue(isinstance(actual, six.text_type))

  def testByString(self):
    module = self.CreateModule('service_module')
    module.package = 'my_package'
    self.assertPackageEquals('my_package',
                             util.get_package_for_module('service_module'))

  def testModuleNameNotInSys(self):
    self.assertPackageEquals(None,
                             util.get_package_for_module('service_module'))

  def testHasPackage(self):
    module = self.CreateModule('service_module')
    module.package = 'my_package'
    self.assertPackageEquals('my_package', util.get_package_for_module(module))

  def testHasModuleName(self):
    module = self.CreateModule('service_module')
    self.assertPackageEquals('service_module',
                             util.get_package_for_module(module))

  def testIsMain(self):
    module = self.CreateModule('__main__')
    module.__file__ = '/bing/blam/bloom/blarm/my_file.py'
    self.assertPackageEquals('my_file', util.get_package_for_module(module))

  def testIsMainCompiled(self):
    module = self.CreateModule('__main__')
    module.__file__ = '/bing/blam/bloom/blarm/my_file.pyc'
    self.assertPackageEquals('my_file', util.get_package_for_module(module))

  def testNoExtension(self):
    module = self.CreateModule('__main__')
    module.__file__ = '/bing/blam/bloom/blarm/my_file'
    self.assertPackageEquals('my_file', util.get_package_for_module(module))

  def testNoPackageAtAll(self):
    module = self.CreateModule('__main__')
    self.assertPackageEquals('__main__', util.get_package_for_module(module))


class DateTimeTests(test_util.TestCase):

  def testDecodeDateTime(self):
    """Test that a RFC 3339 datetime string is decoded properly."""
    for datetime_string, datetime_vals in (
        ('2012-09-30T15:31:50.262', (2012, 9, 30, 15, 31, 50, 262000)),
        ('2012-09-30T15:31:50', (2012, 9, 30, 15, 31, 50, 0))):
      decoded = util.decode_datetime(datetime_string)
      expected = datetime.datetime(*datetime_vals)
      self.assertEquals(expected, decoded)

  def testDateTimeTimeZones(self):
    """Test that a datetime string with a timezone is decoded correctly."""
    for datetime_string, datetime_vals in (
        ('2012-09-30T15:31:50.262-06:00',
         (2012, 9, 30, 15, 31, 50, 262000, util.TimeZoneOffset(-360))),
        ('2012-09-30T15:31:50.262+01:30',
         (2012, 9, 30, 15, 31, 50, 262000, util.TimeZoneOffset(90))),
        ('2012-09-30T15:31:50+00:05',
         (2012, 9, 30, 15, 31, 50, 0, util.TimeZoneOffset(5))),
        ('2012-09-30T15:31:50+00:00',
         (2012, 9, 30, 15, 31, 50, 0, util.TimeZoneOffset(0))),
        ('2012-09-30t15:31:50-00:00',
         (2012, 9, 30, 15, 31, 50, 0, util.TimeZoneOffset(0))),
        ('2012-09-30t15:31:50z',
         (2012, 9, 30, 15, 31, 50, 0, util.TimeZoneOffset(0))),
        ('2012-09-30T15:31:50-23:00',
         (2012, 9, 30, 15, 31, 50, 0, util.TimeZoneOffset(-1380)))):
      decoded = util.decode_datetime(datetime_string)
      expected = datetime.datetime(*datetime_vals)
      self.assertEquals(expected, decoded)

  def testDecodeDateTimeInvalid(self):
    """Test that decoding malformed datetime strings raises execptions."""
    for datetime_string in ('invalid',
                            '2012-09-30T15:31:50.',
                            '-08:00 2012-09-30T15:31:50.262',
                            '2012-09-30T15:31',
                            '2012-09-30T15:31Z',
                            '2012-09-30T15:31:50ZZ',
                            '2012-09-30T15:31:50.262 blah blah -08:00',
                            '1000-99-99T25:99:99.999-99:99'):
      self.assertRaises(ValueError, util.decode_datetime, datetime_string)

  def testTimeZoneOffsetDelta(self):
    """Test that delta works with TimeZoneOffset."""
    time_zone = util.TimeZoneOffset(datetime.timedelta(minutes=3))
    epoch = time_zone.utcoffset(datetime.datetime.utcfromtimestamp(0))
    self.assertEqual(180, util.total_seconds(epoch))


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Stub library."""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import sys
import types

from . import descriptor
from . import message_types
from . import messages
from . import protobuf
from . import remote
from . import util

__all__ = [
    'define_enum',
    'define_field',
    'define_file',
    'define_message',
    'define_service',
    'import_file',
    'import_file_set',
]


# Map variant back to message field classes.
def _build_variant_map():
  """Map variants to fields.

  Returns:
    Dictionary mapping field variant to its associated field type.
  """
  result = {}
  for name in dir(messages):
    value = getattr(messages, name)
    if isinstance(value, type) and issubclass(value, messages.Field):
      for variant in getattr(value, 'VARIANTS', []):
        result[variant] = value
  return result

_VARIANT_MAP = _build_variant_map()

_MESSAGE_TYPE_MAP = {
  message_types.DateTimeMessage.definition_name(): message_types.DateTimeField,
}


def _get_or_define_module(full_name, modules):
  """Helper method for defining new modules.

  Args:
    full_name: Fully qualified name of module to create or return.
    modules: Dictionary of all modules.  Defaults to sys.modules.

  Returns:
    Named module if found in 'modules', else creates new module and inserts in
    'modules'.  Will also construct parent modules if necessary.
  """
  module = modules.get(full_name)
  if not module:
    module = types.ModuleType(full_name)
    modules[full_name] = module

    split_name = full_name.rsplit('.', 1)
    if len(split_name) > 1:
      parent_module_name, sub_module_name = split_name
      parent_module = _get_or_define_module(parent_module_name, modules)
      setattr(parent_module, sub_module_name, module)

  return module


def define_enum(enum_descriptor, module_name):
  """Define Enum class from descriptor.

  Args:
    enum_descriptor: EnumDescriptor to build Enum class from.
    module_name: Module name to give new descriptor class.

  Returns:
    New messages.Enum sub-class as described by enum_descriptor.
  """
  enum_values = enum_descriptor.values or []

  class_dict = dict((value.name, value.number) for value in enum_values)
  class_dict['__module__'] = module_name
  return type(str(enum_descriptor.name), (messages.Enum,), class_dict)


def define_field(field_descriptor):
  """Define Field instance from descriptor.

  Args:
    field_descriptor: FieldDescriptor class to build field instance from.

  Returns:
    New field instance as described by enum_descriptor.
  """
  field_class = _VARIANT_MAP[field_descriptor.variant]
  params = {'number': field_descriptor.number,
            'variant': field_descriptor.variant,
           }

  if field_descriptor.label == descriptor.FieldDescriptor.Label.REQUIRED:
    params['required'] = True
  elif field_descriptor.label == descriptor.FieldDescriptor.Label.REPEATED:
    params['repeated'] = True

  message_type_field = _MESSAGE_TYPE_MAP.get(field_descriptor.type_name)
  if message_type_field:
    return message_type_field(**params)
  elif field_class in (messages.EnumField, messages.MessageField):
    return field_class(field_descriptor.type_name, **params)
  else:
    if field_descriptor.default_value:
      value = field_descriptor.default_value
      try:
        value = descriptor._DEFAULT_FROM_STRING_MAP[field_class](value)
      except (TypeError, ValueError, KeyError):
        pass  # Let the value pass to the constructor.
      params['default'] = value
    return field_class(**params)


def define_message(message_descriptor, module_name):
  """Define Message class from descriptor.

  Args:
    message_descriptor: MessageDescriptor to describe message class from.
    module_name: Module name to give to new descriptor class.

  Returns:
    New messages.Message sub-class as described by message_descriptor.
  """
  class_dict = {'__module__': module_name}

  for enum in message_descriptor.enum_types or []:
    enum_instance = define_enum(enum, module_name)
    class_dict[enum.name] = enum_instance

  # TODO(rafek): support nested messages when supported by descriptor.

  for field in message_descriptor.fields or []:
    field_instance = define_field(field)
    class_dict[field.name] = field_instance

  class_name = message_descriptor.name.encode('utf-8')
  return type(class_name, (messages.Message,), class_dict)


def define_service(service_descriptor, module):
  """Define a new service proxy.

  Args:
    service_descriptor: ServiceDescriptor class that describes the service.
    module: Module to add service to.  Request and response types are found
      relative to this module.

  Returns:
    Service class proxy capable of communicating with a remote server.
  """
  class_dict = {'__module__': module.__name__}
  class_name = service_descriptor.name.encode('utf-8')

  for method_descriptor in service_descriptor.methods or []:
    request_definition = messages.find_definition(
        method_descriptor.request_type, module)
    response_definition = messages.find_definition(
        method_descriptor.response_type, module)

    method_name = method_descriptor.name.encode('utf-8')
    def remote_method(self, request):
      """Actual service method."""
      raise NotImplementedError('Method is not implemented')
    remote_method.__name__ = method_name
    remote_method_decorator = remote.method(request_definition,
                                            response_definition)

    class_dict[method_name] = remote_method_decorator(remote_method)

  service_class = type(class_name, (remote.Service,), class_dict)
  return service_class


def define_file(file_descriptor, module=None):
  """Define module from FileDescriptor.

  Args:
    file_descriptor: FileDescriptor instance to describe module from.
    module: Module to add contained objects to.  Module name overrides value
      in file_descriptor.package.  Definitions are added to existing
      module if provided.

  Returns:
    If no module provided, will create a new module with its name set to the
    file descriptor's package.  If a module is provided, returns the same
    module.
  """
  if module is None:
    module = types.ModuleType(file_descriptor.package)

  for enum_descriptor in file_descriptor.enum_types or []:
    enum_class = define_enum(enum_descriptor, module.__name__)
    setattr(module, enum_descriptor.name, enum_class)

  for message_descriptor in file_descriptor.message_types or []:
    message_class = define_message(message_descriptor, module.__name__)
    setattr(module, message_descriptor.name, message_class)

  for service_descriptor in file_descriptor.service_types or []:
    service_class = define_service(service_descriptor, module)
    setattr(module, service_descriptor.name, service_class)

  return module


@util.positional(1)
def import_file(file_descriptor, modules=None):
  """Import FileDescriptor in to module space.

  This is like define_file except that a new module and any required parent
  modules are created and added to the modules parameter or sys.modules if not
  provided.

  Args:
    file_descriptor: FileDescriptor instance to describe module from.
    modules: Dictionary of modules to update.  Modules and their parents that
      do not exist will be created.  If an existing module is found that
      matches file_descriptor.package, that module is updated with the
      FileDescriptor contents.

  Returns:
    Module found in modules, else a new module.
  """
  if not file_descriptor.package:
    raise ValueError('File descriptor must have package name')

  if modules is None:
    modules = sys.modules

  module = _get_or_define_module(file_descriptor.package.encode('utf-8'),
                                 modules)

  return define_file(file_descriptor, module)


@util.positional(1)
def import_file_set(file_set, modules=None, _open=open):
  """Import FileSet in to module space.

  Args:
    file_set: If string, open file and read serialized FileSet.  Otherwise,
      a FileSet instance to import definitions from.
    modules: Dictionary of modules to update.  Modules and their parents that
      do not exist will be created.  If an existing module is found that
      matches file_descriptor.package, that module is updated with the
      FileDescriptor contents.
    _open: Used for dependency injection during tests.
  """
  if isinstance(file_set, six.string_types):
    encoded_file = _open(file_set, 'rb')
    try:
      encoded_file_set = encoded_file.read()
    finally:
      encoded_file.close()

    file_set = protobuf.decode_message(descriptor.FileSet, encoded_file_set)

  for file_descriptor in file_set.files:
    # Do not reload built in protorpc classes.
    if not file_descriptor.package.startswith('protorpc.'):
      import_file(file_descriptor, modules=modules)



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import with_statement

__author__ = 'rafek@google.com (Rafe Kaplan)'

from . import descriptor
from . import generate
from . import message_types
from . import messages
from . import util


__all__ = ['format_python_file']

_MESSAGE_FIELD_MAP = {
    message_types.DateTimeMessage.definition_name(): message_types.DateTimeField,
}


def _write_enums(enum_descriptors, out):
  """Write nested and non-nested Enum types.

  Args:
    enum_descriptors: List of EnumDescriptor objects from which to generate
      enums.
    out: Indent writer used for generating text.
  """
  # Write enums.
  for enum in enum_descriptors or []:
    out << ''
    out << ''
    out << 'class %s(messages.Enum):' % enum.name
    out << ''

    with out.indent():
      if not enum.values:
        out << 'pass'
      else:
        for enum_value in enum.values:
          out << '%s = %s' % (enum_value.name, enum_value.number)


def _write_fields(field_descriptors, out):
  """Write fields for Message types.

  Args:
    field_descriptors: List of FieldDescriptor objects from which to generate
      fields.
    out: Indent writer used for generating text.
  """
  out << ''
  for field in field_descriptors or []:
    type_format = ''
    label_format = ''

    message_field = _MESSAGE_FIELD_MAP.get(field.type_name)
    if message_field:
      module = 'message_types'
      field_type = message_field
    else:
      module = 'messages'
      field_type = messages.Field.lookup_field_type_by_variant(field.variant)

    if field_type in (messages.EnumField, messages.MessageField):
      type_format = '\'%s\', ' % field.type_name

    if field.label == descriptor.FieldDescriptor.Label.REQUIRED:
      label_format = ', required=True'

    elif field.label == descriptor.FieldDescriptor.Label.REPEATED:
      label_format = ', repeated=True'

    if field_type.DEFAULT_VARIANT != field.variant:
      variant_format = ', variant=messages.Variant.%s' % field.variant
    else:
      variant_format = ''

    if field.default_value:
      if field_type in [messages.BytesField,
                        messages.StringField,
                       ]:
        default_value = repr(field.default_value)
      elif field_type is messages.EnumField:
        try:
          default_value = str(int(field.default_value))
        except ValueError:
          default_value = repr(field.default_value)
      else:
        default_value = field.default_value

      default_format = ', default=%s' % (default_value,)
    else:
      default_format = ''

    out << '%s = %s.%s(%s%s%s%s%s)' % (field.name,
                                       module,
                                       field_type.__name__,
                                       type_format,
                                       field.number,
                                       label_format,
                                       variant_format,
                                       default_format)


def _write_messages(message_descriptors, out):
  """Write nested and non-nested Message types.

  Args:
    message_descriptors: List of MessageDescriptor objects from which to
      generate messages.
    out: Indent writer used for generating text.
  """
  for message in message_descriptors or []:
    out << ''
    out << ''
    out << 'class %s(messages.Message):' % message.name

    with out.indent():
      if not (message.enum_types or message.message_types or message.fields):
        out << ''
        out << 'pass'
      else:
        _write_enums(message.enum_types, out)
        _write_messages(message.message_types, out)
        _write_fields(message.fields, out)


def _write_methods(method_descriptors, out):
  """Write methods of Service types.

  All service method implementations raise NotImplementedError.

  Args:
    method_descriptors: List of MethodDescriptor objects from which to
      generate methods.
    out: Indent writer used for generating text.
  """
  for method in method_descriptors:
    out << ''
    out << "@remote.method('%s', '%s')" % (method.request_type,
                                           method.response_type)
    out << 'def %s(self, request):' % (method.name,)
    with out.indent():
      out << ('raise NotImplementedError'
              "('Method %s is not implemented')" % (method.name))


def _write_services(service_descriptors, out):
  """Write Service types.

  Args:
    service_descriptors: List of ServiceDescriptor instances from which to
      generate services.
    out: Indent writer used for generating text.
  """
  for service in service_descriptors or []:
    out << ''
    out << ''
    out << 'class %s(remote.Service):' % service.name

    with out.indent():
      if service.methods:
        _write_methods(service.methods, out)
      else:
        out << ''
        out << 'pass'


@util.positional(2)
def format_python_file(file_descriptor, output, indent_space=2):
  """Format FileDescriptor object as a single Python module.

  Services generated by this function will raise NotImplementedError.

  All Python classes generated by this function use delayed binding for all
  message fields, enum fields and method parameter types.  For example a
  service method might be generated like so:

    class MyService(remote.Service):

      @remote.method('my_package.MyRequestType', 'my_package.MyResponseType')
      def my_method(self, request):
        raise NotImplementedError('Method my_method is not implemented')

  Args:
    file_descriptor: FileDescriptor instance to format as python module.
    output: File-like object to write module source code to.
    indent_space: Number of spaces for each level of Python indentation.
  """
  out = generate.IndentWriter(output, indent_space=indent_space)

  out << 'from protorpc import message_types'
  out << 'from protorpc import messages'
  if file_descriptor.service_types:
    out << 'from protorpc import remote'

  if file_descriptor.package:
    out << "package = '%s'" % file_descriptor.package

  _write_enums(file_descriptor.enum_types, out)
  _write_messages(file_descriptor.message_types, out)
  _write_services(file_descriptor.service_types, out)



#!/usr/bin/env python
#
# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#




import array
import http.client
import re
import struct

__all__ = ['ProtocolMessage', 'Encoder', 'Decoder',
           'ExtendableProtocolMessage',
           'ProtocolBufferDecodeError',
           'ProtocolBufferEncodeError',
           'ProtocolBufferReturnError']


URL_RE = re.compile('^(https?)://([^/]+)(/.*)$')

class ProtocolMessage:





  def __init__(self, contents=None):
    raise NotImplementedError

  def Clear(self):
    raise NotImplementedError

  def IsInitialized(self, debug_strs=None):
    raise NotImplementedError

  def Encode(self):
    try:
      return self._CEncode()
    except NotImplementedError:
      e = Encoder()
      self.Output(e)
      return e.buffer().tostring()

  def SerializeToString(self):
    return self.Encode()

  def SerializePartialToString(self):
    try:
      return self._CEncodePartial()
    except (NotImplementedError, AttributeError):
      e = Encoder()
      self.OutputPartial(e)
      return e.buffer().tostring()

  def _CEncode(self):
    raise NotImplementedError

  def _CEncodePartial(self):
    raise NotImplementedError

  def ParseFromString(self, s):
    self.Clear()
    self.MergeFromString(s)

  def ParsePartialFromString(self, s):
    self.Clear()
    self.MergePartialFromString(s)

  def MergeFromString(self, s):
    self.MergePartialFromString(s)
    dbg = []
    if not self.IsInitialized(dbg):
      raise ProtocolBufferDecodeError('\n\t'.join(dbg))

  def MergePartialFromString(self, s):
    try:
      self._CMergeFromString(s)
    except NotImplementedError:


      a = array.array('B')
      a.fromstring(s)
      d = Decoder(a, 0, len(a))
      self.TryMerge(d)

  def _CMergeFromString(self, s):
    raise NotImplementedError

  def __getstate__(self):
    return self.Encode()

  def __setstate__(self, contents_):
    self.__init__(contents=contents_)

  def sendCommand(self, server, url, response, follow_redirects=1,
                  secure=0, keyfile=None, certfile=None):
    data = self.Encode()
    if secure:
      if keyfile and certfile:
        conn = http.client.HTTPSConnection(server, key_file=keyfile,
                                       cert_file=certfile)
      else:
        conn = http.client.HTTPSConnection(server)
    else:
      conn = http.client.HTTPConnection(server)
    conn.putrequest("POST", url)
    conn.putheader("Content-Length", str(len(data)))
    conn.endheaders()
    conn.send(data)
    resp = conn.getresponse()
    if follow_redirects > 0 and resp.status == 302:
      m = URL_RE.match(resp.getheader('Location'))
      if m:
        protocol, server, url = m.groups()
        return self.sendCommand(server, url, response,
                                follow_redirects=follow_redirects - 1,
                                secure=(protocol == 'https'),
                                keyfile=keyfile,
                                certfile=certfile)
    if resp.status != 200:
      raise ProtocolBufferReturnError(resp.status)
    if response is not None:
      response.ParseFromString(resp.read())
    return response

  def sendSecureCommand(self, server, keyfile, certfile, url, response,
                        follow_redirects=1):
    return self.sendCommand(server, url, response,
                            follow_redirects=follow_redirects,
                            secure=1, keyfile=keyfile, certfile=certfile)

  def __str__(self, prefix="", printElemNumber=0):
    raise NotImplementedError

  def ToASCII(self):
    return self._CToASCII(ProtocolMessage._SYMBOLIC_FULL_ASCII)

  def ToCompactASCII(self):
    return self._CToASCII(ProtocolMessage._NUMERIC_ASCII)

  def ToShortASCII(self):
    return self._CToASCII(ProtocolMessage._SYMBOLIC_SHORT_ASCII)



  _NUMERIC_ASCII = 0
  _SYMBOLIC_SHORT_ASCII = 1
  _SYMBOLIC_FULL_ASCII = 2

  def _CToASCII(self, output_format):
    raise NotImplementedError

  def ParseASCII(self, ascii_string):
    raise NotImplementedError

  def ParseASCIIIgnoreUnknown(self, ascii_string):
    raise NotImplementedError

  def Equals(self, other):
    raise NotImplementedError

  def __eq__(self, other):





    if other.__class__ is self.__class__:
      return self.Equals(other)
    return NotImplemented

  def __ne__(self, other):





    if other.__class__ is self.__class__:
      return not self.Equals(other)
    return NotImplemented





  def Output(self, e):
    dbg = []
    if not self.IsInitialized(dbg):
      raise ProtocolBufferEncodeError('\n\t'.join(dbg))
    self.OutputUnchecked(e)
    return

  def OutputUnchecked(self, e):
    raise NotImplementedError

  def OutputPartial(self, e):
    raise NotImplementedError

  def Parse(self, d):
    self.Clear()
    self.Merge(d)
    return

  def Merge(self, d):
    self.TryMerge(d)
    dbg = []
    if not self.IsInitialized(dbg):
      raise ProtocolBufferDecodeError('\n\t'.join(dbg))
    return

  def TryMerge(self, d):
    raise NotImplementedError

  def CopyFrom(self, pb):
    if (pb == self): return
    self.Clear()
    self.MergeFrom(pb)

  def MergeFrom(self, pb):
    raise NotImplementedError





  def lengthVarInt32(self, n):
    return self.lengthVarInt64(n)

  def lengthVarInt64(self, n):
    if n < 0:
      return 10
    result = 0
    while 1:
      result += 1
      n >>= 7
      if n == 0:
        break
    return result

  def lengthString(self, n):
    return self.lengthVarInt32(n) + n

  def DebugFormat(self, value):
    return "%s" % value
  def DebugFormatInt32(self, value):
    if (value <= -2000000000 or value >= 2000000000):
      return self.DebugFormatFixed32(value)
    return "%d" % value
  def DebugFormatInt64(self, value):
    if (value <= -20000000000000 or value >= 20000000000000):
      return self.DebugFormatFixed64(value)
    return "%d" % value
  def DebugFormatString(self, value):



    def escape(c):
      o = ord(c)
      if o == 10: return r"\n"
      if o == 39: return r"\'"

      if o == 34: return r'\"'
      if o == 92: return r"\\"

      if o >= 127 or o < 32: return "\\%03o" % o
      return c
    return '"' + "".join([escape(c) for c in value]) + '"'
  def DebugFormatFloat(self, value):
    return "%ff" % value
  def DebugFormatFixed32(self, value):
    if (value < 0): value += (1<<32)
    return "0x%x" % value
  def DebugFormatFixed64(self, value):
    if (value < 0): value += (1<<64)
    return "0x%x" % value
  def DebugFormatBool(self, value):
    if value:
      return "true"
    else:
      return "false"


TYPE_DOUBLE  = 1
TYPE_FLOAT   = 2
TYPE_INT64   = 3
TYPE_UINT64  = 4
TYPE_INT32   = 5
TYPE_FIXED64 = 6
TYPE_FIXED32 = 7
TYPE_BOOL    = 8
TYPE_STRING  = 9
TYPE_GROUP   = 10
TYPE_FOREIGN = 11


_TYPE_TO_DEBUG_STRING = {
    TYPE_INT32:   ProtocolMessage.DebugFormatInt32,
    TYPE_INT64:   ProtocolMessage.DebugFormatInt64,
    TYPE_UINT64:  ProtocolMessage.DebugFormatInt64,
    TYPE_FLOAT:   ProtocolMessage.DebugFormatFloat,
    TYPE_STRING:  ProtocolMessage.DebugFormatString,
    TYPE_FIXED32: ProtocolMessage.DebugFormatFixed32,
    TYPE_FIXED64: ProtocolMessage.DebugFormatFixed64,
    TYPE_BOOL:    ProtocolMessage.DebugFormatBool }



class Encoder:


  NUMERIC     = 0
  DOUBLE      = 1
  STRING      = 2
  STARTGROUP  = 3
  ENDGROUP    = 4
  FLOAT       = 5
  MAX_TYPE    = 6

  def __init__(self):
    self.buf = array.array('B')
    return

  def buffer(self):
    return self.buf

  def put8(self, v):
    if v < 0 or v >= (1<<8): raise ProtocolBufferEncodeError("u8 too big")
    self.buf.append(v & 255)
    return

  def put16(self, v):
    if v < 0 or v >= (1<<16): raise ProtocolBufferEncodeError("u16 too big")
    self.buf.append((v >> 0) & 255)
    self.buf.append((v >> 8) & 255)
    return

  def put32(self, v):
    if v < 0 or v >= (1<<32): raise ProtocolBufferEncodeError("u32 too big")
    self.buf.append((v >> 0) & 255)
    self.buf.append((v >> 8) & 255)
    self.buf.append((v >> 16) & 255)
    self.buf.append((v >> 24) & 255)
    return

  def put64(self, v):
    if v < 0 or v >= (1<<64): raise ProtocolBufferEncodeError("u64 too big")
    self.buf.append((v >> 0) & 255)
    self.buf.append((v >> 8) & 255)
    self.buf.append((v >> 16) & 255)
    self.buf.append((v >> 24) & 255)
    self.buf.append((v >> 32) & 255)
    self.buf.append((v >> 40) & 255)
    self.buf.append((v >> 48) & 255)
    self.buf.append((v >> 56) & 255)
    return

  def putVarInt32(self, v):








    buf_append = self.buf.append
    if v & 127 == v:
      buf_append(v)
      return
    if v >= 0x80000000 or v < -0x80000000:
      raise ProtocolBufferEncodeError("int32 too big")
    if v < 0:
      v += 0x10000000000000000
    while True:
      bits = v & 127
      v >>= 7
      if v:
        bits |= 128
      buf_append(bits)
      if not v:
        break
    return

  def putVarInt64(self, v):
    buf_append = self.buf.append
    if v >= 0x8000000000000000 or v < -0x8000000000000000:
      raise ProtocolBufferEncodeError("int64 too big")
    if v < 0:
      v += 0x10000000000000000
    while True:
      bits = v & 127
      v >>= 7
      if v:
        bits |= 128
      buf_append(bits)
      if not v:
        break
    return

  def putVarUint64(self, v):
    buf_append = self.buf.append
    if v < 0 or v >= 0x10000000000000000:
      raise ProtocolBufferEncodeError("uint64 too big")
    while True:
      bits = v & 127
      v >>= 7
      if v:
        bits |= 128
      buf_append(bits)
      if not v:
        break
    return






  def putFloat(self, v):
    a = array.array('B')
    a.fromstring(struct.pack("<f", v))
    self.buf.extend(a)
    return

  def putDouble(self, v):
    a = array.array('B')
    a.fromstring(struct.pack("<d", v))
    self.buf.extend(a)
    return

  def putBoolean(self, v):
    if v:
      self.buf.append(1)
    else:
      self.buf.append(0)
    return

  def putPrefixedString(self, v):



    v = str(v)
    self.putVarInt32(len(v))
    self.buf.fromstring(v)
    return

  def putRawString(self, v):
    self.buf.fromstring(v)

  _TYPE_TO_METHOD = {
      TYPE_DOUBLE:   putDouble,
      TYPE_FLOAT:    putFloat,
      TYPE_FIXED64:  put64,
      TYPE_FIXED32:  put32,
      TYPE_INT32:    putVarInt32,
      TYPE_INT64:    putVarInt64,
      TYPE_UINT64:   putVarUint64,
      TYPE_BOOL:     putBoolean,
      TYPE_STRING:   putPrefixedString }

  _TYPE_TO_BYTE_SIZE = {
      TYPE_DOUBLE:  8,
      TYPE_FLOAT:   4,
      TYPE_FIXED64: 8,
      TYPE_FIXED32: 4,
      TYPE_BOOL:    1 }

class Decoder:
  def __init__(self, buf, idx, limit):
    self.buf = buf
    self.idx = idx
    self.limit = limit
    return

  def avail(self):
    return self.limit - self.idx

  def buffer(self):
    return self.buf

  def pos(self):
    return self.idx

  def skip(self, n):
    if self.idx + n > self.limit: raise ProtocolBufferDecodeError("truncated")
    self.idx += n
    return

  def skipData(self, tag):
    t = tag & 7
    if t == Encoder.NUMERIC:
      self.getVarInt64()
    elif t == Encoder.DOUBLE:
      self.skip(8)
    elif t == Encoder.STRING:
      n = self.getVarInt32()
      self.skip(n)
    elif t == Encoder.STARTGROUP:
      while 1:
        t = self.getVarInt32()
        if (t & 7) == Encoder.ENDGROUP:
          break
        else:
          self.skipData(t)
      if (t - Encoder.ENDGROUP) != (tag - Encoder.STARTGROUP):
        raise ProtocolBufferDecodeError("corrupted")
    elif t == Encoder.ENDGROUP:
      raise ProtocolBufferDecodeError("corrupted")
    elif t == Encoder.FLOAT:
      self.skip(4)
    else:
      raise ProtocolBufferDecodeError("corrupted")


  def get8(self):
    if self.idx >= self.limit: raise ProtocolBufferDecodeError("truncated")
    c = self.buf[self.idx]
    self.idx += 1
    return c

  def get16(self):
    if self.idx + 2 > self.limit: raise ProtocolBufferDecodeError("truncated")
    c = self.buf[self.idx]
    d = self.buf[self.idx + 1]
    self.idx += 2
    return (d << 8) | c

  def get32(self):
    if self.idx + 4 > self.limit: raise ProtocolBufferDecodeError("truncated")
    c = self.buf[self.idx]
    d = self.buf[self.idx + 1]
    e = self.buf[self.idx + 2]
    f = int(self.buf[self.idx + 3])
    self.idx += 4
    return (f << 24) | (e << 16) | (d << 8) | c

  def get64(self):
    if self.idx + 8 > self.limit: raise ProtocolBufferDecodeError("truncated")
    c = self.buf[self.idx]
    d = self.buf[self.idx + 1]
    e = self.buf[self.idx + 2]
    f = int(self.buf[self.idx + 3])
    g = int(self.buf[self.idx + 4])
    h = int(self.buf[self.idx + 5])
    i = int(self.buf[self.idx + 6])
    j = int(self.buf[self.idx + 7])
    self.idx += 8
    return ((j << 56) | (i << 48) | (h << 40) | (g << 32) | (f << 24)
            | (e << 16) | (d << 8) | c)

  def getVarInt32(self):



    b = self.get8()
    if not (b & 128):
      return b

    result = int(0)
    shift = 0

    while 1:
      result |= (int(b & 127) << shift)
      shift += 7
      if not (b & 128):
        if result >= 0x10000000000000000:
          raise ProtocolBufferDecodeError("corrupted")
        break
      if shift >= 64: raise ProtocolBufferDecodeError("corrupted")
      b = self.get8()

    if result >= 0x8000000000000000:
      result -= 0x10000000000000000
    if result >= 0x80000000 or result < -0x80000000:
      raise ProtocolBufferDecodeError("corrupted")
    return result

  def getVarInt64(self):
    result = self.getVarUint64()
    if result >= (1 << 63):
      result -= (1 << 64)
    return result

  def getVarUint64(self):
    result = int(0)
    shift = 0
    while 1:
      if shift >= 64: raise ProtocolBufferDecodeError("corrupted")
      b = self.get8()
      result |= (int(b & 127) << shift)
      shift += 7
      if not (b & 128):
        if result >= (1 << 64): raise ProtocolBufferDecodeError("corrupted")
        return result
    return result

  def getFloat(self):
    if self.idx + 4 > self.limit: raise ProtocolBufferDecodeError("truncated")
    a = self.buf[self.idx:self.idx+4]
    self.idx += 4
    return struct.unpack("<f", a)[0]

  def getDouble(self):
    if self.idx + 8 > self.limit: raise ProtocolBufferDecodeError("truncated")
    a = self.buf[self.idx:self.idx+8]
    self.idx += 8
    return struct.unpack("<d", a)[0]

  def getBoolean(self):
    b = self.get8()
    if b != 0 and b != 1: raise ProtocolBufferDecodeError("corrupted")
    return b

  def getPrefixedString(self):
    length = self.getVarInt32()
    if self.idx + length > self.limit:
      raise ProtocolBufferDecodeError("truncated")
    r = self.buf[self.idx : self.idx + length]
    self.idx += length
    return r.tostring()

  def getRawString(self):
    r = self.buf[self.idx:self.limit]
    self.idx = self.limit
    return r.tostring()

  _TYPE_TO_METHOD = {
      TYPE_DOUBLE:   getDouble,
      TYPE_FLOAT:    getFloat,
      TYPE_FIXED64:  get64,
      TYPE_FIXED32:  get32,
      TYPE_INT32:    getVarInt32,
      TYPE_INT64:    getVarInt64,
      TYPE_UINT64:   getVarUint64,
      TYPE_BOOL:     getBoolean,
      TYPE_STRING:   getPrefixedString }





class ExtensionIdentifier(object):
  __slots__ = ('full_name', 'number', 'field_type', 'wire_tag', 'is_repeated',
               'default', 'containing_cls', 'composite_cls', 'message_name')
  def __init__(self, full_name, number, field_type, wire_tag, is_repeated,
               default):
    self.full_name = full_name
    self.number = number
    self.field_type = field_type
    self.wire_tag = wire_tag
    self.is_repeated = is_repeated
    self.default = default

class ExtendableProtocolMessage(ProtocolMessage):
  def HasExtension(self, extension):
    self._VerifyExtensionIdentifier(extension)
    return extension in self._extension_fields

  def ClearExtension(self, extension):
    self._VerifyExtensionIdentifier(extension)
    if extension in self._extension_fields:
      del self._extension_fields[extension]

  def GetExtension(self, extension, index=None):
    self._VerifyExtensionIdentifier(extension)
    if extension in self._extension_fields:
      result = self._extension_fields[extension]
    else:
      if extension.is_repeated:
        result = []
      elif extension.composite_cls:
        result = extension.composite_cls()
      else:
        result = extension.default
    if extension.is_repeated:
      result = result[index]
    return result

  def SetExtension(self, extension, *args):
    self._VerifyExtensionIdentifier(extension)
    if extension.composite_cls:
      raise TypeError(
          'Cannot assign to extension "%s" because it is a composite type.' %
          extension.full_name)
    if extension.is_repeated:
      if (len(args) != 2):
        raise TypeError(
            'SetExtension(extension, index, value) for repeated extension '
            'takes exactly 3 arguments: (%d given)' % len(args))
      index = args[0]
      value = args[1]
      self._extension_fields[extension][index] = value
    else:
      if (len(args) != 1):
        raise TypeError(
            'SetExtension(extension, value) for singular extension '
            'takes exactly 3 arguments: (%d given)' % len(args))
      value = args[0]
      self._extension_fields[extension] = value

  def MutableExtension(self, extension, index=None):
    self._VerifyExtensionIdentifier(extension)
    if extension.composite_cls is None:
      raise TypeError(
          'MutableExtension() cannot be applied to "%s", because it is not a '
          'composite type.' % extension.full_name)
    if extension.is_repeated:
      if index is None:
        raise TypeError(
            'MutableExtension(extension, index) for repeated extension '
            'takes exactly 2 arguments: (1 given)')
      return self.GetExtension(extension, index)
    if extension in self._extension_fields:
      return self._extension_fields[extension]
    else:
      result = extension.composite_cls()
      self._extension_fields[extension] = result
      return result

  def ExtensionList(self, extension):
    self._VerifyExtensionIdentifier(extension)
    if not extension.is_repeated:
      raise TypeError(
          'ExtensionList() cannot be applied to "%s", because it is not a '
          'repeated extension.' % extension.full_name)
    if extension in self._extension_fields:
      return self._extension_fields[extension]
    result = []
    self._extension_fields[extension] = result
    return result

  def ExtensionSize(self, extension):
    self._VerifyExtensionIdentifier(extension)
    if not extension.is_repeated:
      raise TypeError(
          'ExtensionSize() cannot be applied to "%s", because it is not a '
          'repeated extension.' % extension.full_name)
    if extension in self._extension_fields:
      return len(self._extension_fields[extension])
    return 0

  def AddExtension(self, extension, value=None):
    self._VerifyExtensionIdentifier(extension)
    if not extension.is_repeated:
      raise TypeError(
          'AddExtension() cannot be applied to "%s", because it is not a '
          'repeated extension.' % extension.full_name)
    if extension in self._extension_fields:
      field = self._extension_fields[extension]
    else:
      field = []
      self._extension_fields[extension] = field

    if extension.composite_cls:
      if value is not None:
        raise TypeError(
            'value must not be set in AddExtension() for "%s", because it is '
            'a message type extension. Set values on the returned message '
            'instead.' % extension.full_name)
      msg = extension.composite_cls()
      field.append(msg)
      return msg

    field.append(value)

  def _VerifyExtensionIdentifier(self, extension):
    if extension.containing_cls != self.__class__:
      raise TypeError("Containing type of %s is %s, but not %s."
                      % (extension.full_name,
                         extension.containing_cls.__name__,
                         self.__class__.__name__))

  def _MergeExtensionFields(self, x):
    for ext, val in list(x._extension_fields.items()):
      if ext.is_repeated:
        for i in range(len(val)):
          if ext.composite_cls is None:
            self.AddExtension(ext, val[i])
          else:
            self.AddExtension(ext).MergeFrom(val[i])
      else:
        if ext.composite_cls is None:
          self.SetExtension(ext, val)
        else:
          self.MutableExtension(ext).MergeFrom(val)

  def _ListExtensions(self):
    result = [ext for ext in list(self._extension_fields.keys())
              if (not ext.is_repeated) or self.ExtensionSize(ext) > 0]
    result.sort(key = lambda item: item.number)
    return result

  def _ExtensionEquals(self, x):
    extensions = self._ListExtensions()
    if extensions != x._ListExtensions():
      return False
    for ext in extensions:
      if ext.is_repeated:
        if self.ExtensionSize(ext) != x.ExtensionSize(ext): return False
        for e1, e2 in zip(self.ExtensionList(ext),
                          x.ExtensionList(ext)):
          if e1 != e2: return False
      else:
        if self.GetExtension(ext) != x.GetExtension(ext): return False
    return True

  def _OutputExtensionFields(self, out, partial, extensions, start_index,
                             end_field_number):
    def OutputSingleField(ext, value):
      out.putVarInt32(ext.wire_tag)
      if ext.field_type == TYPE_GROUP:
        if partial:
          value.OutputPartial(out)
        else:
          value.OutputUnchecked(out)
        out.putVarInt32(ext.wire_tag + 1)
      elif ext.field_type == TYPE_FOREIGN:
        if partial:
          out.putVarInt32(value.ByteSizePartial())
          value.OutputPartial(out)
        else:
          out.putVarInt32(value.ByteSize())
          value.OutputUnchecked(out)
      else:
        Encoder._TYPE_TO_METHOD[ext.field_type](out, value)

    size = len(extensions)
    for ext_index in range(start_index, size):
      ext = extensions[ext_index]
      if ext.number >= end_field_number:

        return ext_index
      if ext.is_repeated:
        for i in range(len(self._extension_fields[ext])):
          OutputSingleField(ext, self._extension_fields[ext][i])
      else:
        OutputSingleField(ext, self._extension_fields[ext])
    return size

  def _ParseOneExtensionField(self, wire_tag, d):
    number = wire_tag >> 3
    if number in self._extensions_by_field_number:
      ext = self._extensions_by_field_number[number]
      if wire_tag != ext.wire_tag:

        return
      if ext.field_type == TYPE_FOREIGN:
        length = d.getVarInt32()
        tmp = Decoder(d.buffer(), d.pos(), d.pos() + length)
        if ext.is_repeated:
          self.AddExtension(ext).TryMerge(tmp)
        else:
          self.MutableExtension(ext).TryMerge(tmp)
        d.skip(length)
      elif ext.field_type == TYPE_GROUP:
        if ext.is_repeated:
          self.AddExtension(ext).TryMerge(d)
        else:
          self.MutableExtension(ext).TryMerge(d)
      else:
        value = Decoder._TYPE_TO_METHOD[ext.field_type](d)
        if ext.is_repeated:
          self.AddExtension(ext, value)
        else:
          self.SetExtension(ext, value)
    else:

      d.skipData(wire_tag)

  def _ExtensionByteSize(self, partial):
    size = 0
    for extension, value in list(self._extension_fields.items()):
      ftype = extension.field_type
      tag_size = self.lengthVarInt64(extension.wire_tag)
      if ftype == TYPE_GROUP:
        tag_size *= 2
      if extension.is_repeated:
        size += tag_size * len(value)
        for single_value in value:
          size += self._FieldByteSize(ftype, single_value, partial)
      else:
        size += tag_size + self._FieldByteSize(ftype, value, partial)
    return size

  def _FieldByteSize(self, ftype, value, partial):
    size = 0
    if ftype == TYPE_STRING:
      size = self.lengthString(len(value))
    elif ftype == TYPE_FOREIGN or ftype == TYPE_GROUP:
      if partial:
        size = self.lengthString(value.ByteSizePartial())
      else:
        size = self.lengthString(value.ByteSize())
    elif ftype == TYPE_INT64 or ftype == TYPE_UINT64 or ftype == TYPE_INT32:
      size = self.lengthVarInt64(value)
    else:
      if ftype in Encoder._TYPE_TO_BYTE_SIZE:
        size = Encoder._TYPE_TO_BYTE_SIZE[ftype]
      else:
        raise AssertionError(
            'Extension type %d is not recognized.' % ftype)
    return size

  def _ExtensionDebugString(self, prefix, printElemNumber):
    res = ''
    extensions = self._ListExtensions()
    for extension in extensions:
      value = self._extension_fields[extension]
      if extension.is_repeated:
        cnt = 0
        for e in value:
          elm=""
          if printElemNumber: elm = "(%d)" % cnt
          if extension.composite_cls is not None:
            res += prefix + "[%s%s] {\n" % (extension.full_name, elm)
            res += e.__str__(prefix + "  ", printElemNumber)
            res += prefix + "}\n"
      else:
        if extension.composite_cls is not None:
          res += prefix + "[%s] {\n" % extension.full_name
          res += value.__str__(
              prefix + "  ", printElemNumber)
          res += prefix + "}\n"
        else:
          if extension.field_type in _TYPE_TO_DEBUG_STRING:
            text_value = _TYPE_TO_DEBUG_STRING[
                extension.field_type](self, value)
          else:
            text_value = self.DebugFormat(value)
          res += prefix + "[%s]: %s\n" % (extension.full_name, text_value)
    return res

  @staticmethod
  def _RegisterExtension(cls, extension, composite_cls=None):
    extension.containing_cls = cls
    extension.composite_cls = composite_cls
    if composite_cls is not None:
      extension.message_name = composite_cls._PROTO_DESCRIPTOR_NAME
    actual_handle = cls._extensions_by_field_number.setdefault(
        extension.number, extension)
    if actual_handle is not extension:
      raise AssertionError(
          'Extensions "%s" and "%s" both try to extend message type "%s" with'
          'field number %d.' %
          (extension.full_name, actual_handle.full_name,
           cls.__name__, extension.number))

class ProtocolBufferDecodeError(Exception): pass
class ProtocolBufferEncodeError(Exception): pass
class ProtocolBufferReturnError(Exception): pass
